@@ -1,10 +1,13 @@
 """Utility functions."""
 import uuid
-from typing import Dict, Callable, Union, Type, Tuple, List, Any
+from typing import Dict, Callable, Union, Type, Tuple, List, Any, Optional
+from warnings import warn
 
 from gemd.entity.base_entity import BaseEntity
 from gemd.entity.dict_serializable import DictSerializable
 from gemd.entity.link_by_uid import LinkByUID
+
+from collections.abc import Reversible, Iterable
 from toolz import concatv
 
 
@@ -63,7 +66,8 @@ def _cached_isinstance_generator(
 
 # The overhead for all the invocations of isinstance was substantial
 isinstance_base_entity = _cached_isinstance_generator(BaseEntity)
-isinstance_list_or_tuple = _cached_isinstance_generator((list, tuple))
+isinstance_iterable = _cached_isinstance_generator(Iterable)
+isinstance_reversible = _cached_isinstance_generator(Reversible)
 isinstance_list = _cached_isinstance_generator(list)
 isinstance_tuple = _cached_isinstance_generator(tuple)
 isinstance_dict = _cached_isinstance_generator(dict)
@@ -137,7 +141,7 @@ def make_index(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]):
 
     """
     def _make_index(_obj: BaseEntity):
-        return (((scope, _obj.uids[scope]), _obj) for scope in _obj.uids)
+        return ((LinkByUID(scope=scope, id=_obj.uids[scope]), _obj) for scope in _obj.uids)
 
     idx = {}
     for uid, target in recursive_flatmap(obj, _make_index):
@@ -146,7 +150,12 @@ def make_index(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable]):
     return idx
 
 
-def substitute_links(obj: Any, native_uid=None):
+def substitute_links(obj: Any,
+                     scope: Optional[str] = None,
+                     *,
+                     native_uid: str = None,
+                     allow_fallback: bool = True
+                     ):
     """
     Recursively replace pointers to BaseEntity with LinkByUID objects.
 
@@ -157,20 +166,24 @@ def substitute_links(obj: Any, native_uid=None):
     ----------
     obj: Any
         target of the operation
-    native_uid: Optional[str]
+    scope: Optional[str], optional
         preferred scope to use for creating LinkByUID objects (Default: None)
+    native_uid: str, optional
+        DEPRECATED; former name for scope argument
+    allow_fallback: bool, optional
+        whether to grab another scope/id if chosen scope is missing (Default: True).
 
     """
-    def make_link(entity: BaseEntity):
-        if len(entity.uids) == 0:
-            raise ValueError("No UID for {}".format(entity))
-        elif native_uid and native_uid in entity.uids:
-            return LinkByUID(native_uid, entity.uids[native_uid])
-        else:
-            return LinkByUID.from_entity(entity)
+    if native_uid is not None:
+        warn("The keyword argument 'native_uid' is deprecated.  When selecting a default scope, "
+             "use the 'scope' keyword argument.", DeprecationWarning)
+        if scope is not None:
+            raise ValueError("Both 'scope' and 'native_uid' keywords passed.")
+        scope = native_uid
 
-    return _substitute(obj, sub=make_link,
-                       applies=lambda o: o is not obj and isinstance(o, BaseEntity))
+    return _substitute(obj,
+                       sub=lambda o: o.to_link(scope=scope, allow_fallback=allow_fallback),
+                       applies=lambda o: o is not obj and isinstance_base_entity(o))
 
 
 def substitute_objects(obj, index):
@@ -189,7 +202,7 @@ def substitute_objects(obj, index):
 
     """
     return _substitute(obj,
-                       sub=lambda l: index.get((l.scope.lower(), l.id), l),
+                       sub=lambda l: index.get(l, l),
                        applies=lambda o: isinstance(o, LinkByUID))
 
 
@@ -289,15 +302,15 @@ def recursive_foreach(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable
         if apply_first and isinstance_base_entity(this):
             func(this)
 
-        if isinstance_list_or_tuple(this):
-            for x in this:
-                queue.append(x)
-        elif isinstance_dict(this):
+        if isinstance_dict(this):
             for x in concatv(this.keys(), this.values()):
                 queue.append(x)
         elif isinstance_dict_serializable(this):
             for k, x in this.__dict__.items():
                 queue.append(x)
+        elif isinstance_iterable(this):
+            for x in this:
+                queue.append(x)
 
         if not apply_first and isinstance_base_entity(this):
             func(this)
@@ -343,15 +356,17 @@ def recursive_flatmap(obj: Union[List, Tuple, Dict, BaseEntity, DictSerializable
         if isinstance_base_entity(this):
             res.extend(func(this))
 
-        if isinstance_list_or_tuple(this):
-            queue.extend(reversed(this))  # Preserve order of the list/tuple
-        elif isinstance_dict(this):
+        if isinstance_dict(this):
             queue.extend(concatv(this.keys(), this.values()))
         elif isinstance_dict_serializable(this):
             for k, x in sorted(this.__dict__.items()):
                 if unidirectional and isinstance_base_entity(this) and k in this.skip:
                     continue
                 queue.append(x)
+        elif isinstance_reversible(this):
+            queue.extend(reversed(this))  # Preserve order of the list/tuple
+        elif isinstance_iterable(this):
+            queue.extend(this)  # No control over order
 
     return res
 
@@ -363,7 +378,7 @@ def writable_sort_order(key: Union[BaseEntity, str]) -> int:
     from gemd.entity.template import ConditionTemplate, MaterialTemplate, MeasurementTemplate, \
         ParameterTemplate, ProcessTemplate, PropertyTemplate
 
-    if isinstance(key, BaseEntity):
+    if isinstance_base_entity(key):
         typ = key.typ
     elif isinstance(key, str):
         typ = key
